//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	IntLim
// Auteur			:	PERRICHON Pierre
// Date				:	01/05/2014
// Version			:	1.0
//
// Sujet			: 	Intégrateurs avec réinitialisation et saturations externes
//
// Paramétrage		:	Constante de temps Ti de l'intégrateur (s)
//						Valeur initiale de l'intégrateur à t=0
//						Avec ou sans réinitialisation programmable (0 ou 1)
//						Avec ou sans saturations externes (0 ou 1)
//
// Autres données	:	
//						
//						
// Sorties			:	Intégrateurs avec saturation
//						
//
// Modification		:
// Auteur			:	PERRICHON Pierre
// Date				:	25/08/2021
// Version			:	1.1
//
// Description		!	Réarragement des E/S de la boite fonctionnelles
//						Entrée "in" vectorisée n lignes x 1 colonne
//						Réinitialisation : 1 scalaire 1x1 pour tous les intégrateurs
//						Limitations UL et LL 1 scalire 1x1 pour tous les intégrateurs
//						Sortie out vectorisée n lignes x 1 colonne						
//						
// Interface IHM	:	IntLim.sci
// Code objet		;	IntLimC.c 
//
// Programmes de test:	IntLim.zos Les_Trois_SYSCOP.zcos
//
//------------------------------------------------------------------------------------------------------------------------
function [sciblk, s1, s2]=IntLim(job, sciblk, e1)

  s1= []; s2 = [];// arguments requis mais non utilisés 
  
  	Module="IntLim";
	[linenum, fun] = where()
	editSegPalBlock=~or(fun == "do_eval")	// Détertime si mode simulation ou édition du composant par double clic

  
	select job 
  
		case "set" then 
		
			graphics=sciblk.graphics;
			exprs=graphics.exprs;
			model=sciblk.model;
						
			// Analyse préliminaire des expressions
			//-------------------------------------
			[bdiag,DefExp]=CheckExprs(exprs) 		
			if ~bdiag then 
						// Certaines variables ne sont pas définies
						//-----------------------------------------
						if SegPalError(2, Module, DefExp)==2 then return; end
			end

			Titre="Paramètres de l''intégrateur IntLim"			
			
			labels=["Constante de temps de l''intégrateur (s)";...
			        "Valeur initiale de l''intégrateur à t=0";...
					"Avec réinitialisation programmable (1:oui - 0:non)";...
					"Avec saturation dynamique (1:oui - 0:non)"];
					
			messerr1="La constante de temps de l''intégrateur doit être strictement positive"

			if getlanguage()=="en_US" then		

				Titre="IntLim integrator settings"			
				
				labels=["Integrator time constant (s)";...
			        "Initial value of the integrator at t=0";...
					"With programmable reset (1:yes - 0:no)";...
					"With dynamic saturation (1:yes - 0:no)"];
					
				messerr1="The time constant of the integrator must be strictly positive"
			end
			
								
			while %t do
			
				[ok,Tau,x0,reinit,sature,exprs]=scicos_getvalue(Titre,labels,...
									list("vec",1,"vec",-1,"vec",1,"vec",1),exprs)
					
				if ~ok then return; end
				
				//Analyse du paramétrage
				//----------------------			
				[bdiag,DefExp]=CheckExprs(exprs)
				if ~bdiag then SegPalError(2, Module, DefExp)
					else
							DefExp=[]
							if Tau<=0. then DefExp=[DefExp;messerr1]; end

							if DefExp==[] then break; end
									
							SegPalError(1, Module, DefExp);
				end				
							
			end
				
				
			n = size(x0,'*');	// Donne le dimensionnement variable de la boite
					
			out = [n, 1];
			outype = 1;
			evtout = []; //aucun port de sortie d'activation
					
			intype = 1;
			in = [n, 1];
			graphics.in_label=[""];
			if(reinit + sature) then graphics.in_label=["in"];end;
			
			evtin = [];
										
			// Traitement de la réinitialisation
			//----------------------------------							
			if reinit~=0 then
							reinit=1;					
							// Rajout d'1 entrée régulière de dimension 1 x 1 
							//-----------------------------------------------
							intype=[intype, 1];
							in=[in; 1,1];
							graphics.in_label=[graphics.in_label, "init"];

							// Un port d'entrée d'activation
							//------------------------------
							evtin = 1; 							
			end
					
			//Traitement de la saturation
			//---------------------------
			if sature~=0 then
						sature=1;
						// Rajout de 2 entrées régulières de dimension 1 x 1
						//---------------------------------------------------
						intype=[intype, 1, 1];
						in=[in; 1,1; 1, 1];
						graphics.in_label=[graphics.in_label,"UL","LL"];
			end
					
			//Redessiner l'interface graphique en fonction de la valeur d'initialisation
			//--------------------------------------------------------------------------
			[model,graphics,ok] = set_io(model, graphics, list(in,intype), list(out, outype), evtin, evtout)
				
			// Terminé ==> transfert des structures et validation
			//---------------------------------------------------
			model.state = x0;			// Valeurw initialew des intégrateurs
			model.rpar=[Tau];			// Constante de temps des intégrateurs
			model.ipar=[reinit;sature]							
							
			sciblk.model = model;
			graphics.exprs = exprs;
			sciblk.graphics = graphics;
							
			//Affichage de la constante de temps des intégrateurs dans la boîte fonctionnelle
			//--------------------------------------------------------------------------------
			Taudisplay=string(exprs(1))+ "."
			if strcmp(Taudisplay,"1.")==0 then Taudisplay="";end;
							
			style_properties = ["blockWithLabel";
									"verticalLabelPosition=middle";
									"verticalAlign=top";
									"fontColor=blue";
									"fontSize=15";
									"displayedLabel=$\frac{1}{\ " + Taudisplay + "p\ }$";
									"fillColor=#abcdee";
									"shadow=true"];									
			sciblk.graphics.style = strcat(style_properties,";");
							
	
		case "define" then 
	
			Tau=1;		// Constante de temps du filtre passe-bas (s)
			x0=0;		// Valeur initiale nulle du filtre passe-bas			
			reinit=1;	// Réinitialisation du filtre passe-bas			
			sature=1;	// Saturation dynamique
			
			model = scicos_model();
			
			model.sim=list("IntLimC",4)			// Pour appel fonction "c" IntLimC
						
			model.intyp = [1;1;1;1];
			model.in = [1;1;1;1]; model.in2 = [1;1;1;1]; 		//4 entrées scalaire

			model.outtyp = 1;
			model.out = 1; model.out2 = 1;		//une sortie scalaire

			model.evtin = 1 					// 1 port d'entrée d'activation
			
			model.state = x0;
			
			model.ipar=[reinit;sature]
			model.rpar=[Tau]
			model.blocktype="c"
			
			model.dep_ut = [%t %t];
			
			exprs = [sci2exp(Tau);sci2exp(x0);sci2exp(reinit);sci2exp(sature)];
			
			sciblk = standard_define([3 3],model,exprs);
			
			style_properties = ["blockWithLabel";
                    "verticalLabelPosition=middle";
                    "verticalAlign=top";
					"fontColor=blue";
					"fontSize=15";
                    "displayedLabel=$\frac{1}{\tau.p\ }$";
					"fillColor=#abcdee";
					"shadow=true"];
					
			sciblk.graphics.style = strcat(style_properties,";")	
			
	end
  
endfunction