//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	IntLim
// Auteur			:	PERRICHON Pierre
// Date				:	02/08/2022
// Version			:	1.0
//
// Sujet			: 	Intégrateurs avec désaturateur rapide et saturations externes
//
// Paramétrage		:	Constante de temps Ti de l'intégrateur (s)
//						Valeur initiale de l'intégrateur à t=0
//						Constante de temps du désaturateur(s)
//						Avec ou sans saturations externes (0 ou 1)
//
// Autres données	:	
//						
//						
// Sorties			:	Intégrateurs avec désaturateur rapide et saturations externes
//						
//						
					
// Interface IHM	:	IntSPE.sci
// Code objet		;	IntSPEC.c 
//
// Programmes de test:	IntSPE.zos
//
//------------------------------------------------------------------------------------------------------------------------

#include "Common.h"
#include <sciprint.h>

#include <math.h>
#include <stdio.h>
#include "scicos_block4.h"
//#include "scicos.h"

#include "version.h"
#if SCI_VERSION_MAJOR < 6
#include "MALLOC.h"
#else
#include "sci_malloc.h"
#endif

#include <sciprint.h>


void  IntSPEC(scicos_block *Cblk, scicos_flag flag)
{
	static char *Module="IntSPEC";
	
	char MesErr[120];
	char *Label;
	int NumBlockErr;
	
	int i = 0;
	int n = GetInPortRows(Cblk,1); 
	double IN;
	
	int reinit=Cblk->ipar[0];
	int sature=Cblk->ipar[1];
	double UL,LL;			// Upper Limiter - Lower Limiter - Limiteurs de l'intégrateur

	int numport;
	
	double *in, *in2, *out, *x, *xd, Tau, TauInv;
	int nevptr;
	
	double *work = NULL;
	
	double t;
	
	if (sature) {
					numport=2;
					if (reinit) numport=3;
					 
					UL=*GetRealInPortPtrs(Cblk,numport);
					LL=*GetRealInPortPtrs(Cblk,numport+1);
					
					if (LL>UL) {
						//set_block_error(-1); 	// Données d'entrée inappropriées : voir help c_utils
						NumBlockErr=get_block_number();
						Label=Cblk->label;
						sprintf(MesErr,"Module %s - Block %i \nLower limiter LL=%f greater than the Upper UL=%f",Module,NumBlockErr,LL,UL);
						Coserror(MesErr);
					}
	}

	switch(flag) {
		
		case Initialization:
					// Allocation d'une zône de travail de 2 doubles
					//----------------------------------------------
					if ((GetWorkPtrs(Cblk) = (double *)MALLOC(2*sizeof(double)))== NULL ) {
						Coserror("%s: Memory allocation problem\n", "LocalMax");
						return;
					}					
					work = (double *)GetWorkPtrs(Cblk);
					
					work[0] = 0.0;				// Indicateur de désaturation en cours 
					work[1] = 0.0;				// Indicateur de désaturation en cours 					
					
					break;	
		
	
		case DerivativeState: 
			// Evaluation de la fonction C 
			//----------------------------
			in = GetRealInPortPtrs(Cblk,1);  
			xd = GetDerState(Cblk);
			
			// Détermination de la constante d'intégration
			//--------------------------------------------
			work = (double *)GetWorkPtrs(Cblk);
			Tau= work[0]==0. ? Cblk->rpar[0] : -Cblk->rpar[1];
			IN= work[0]==0. ? in[0] : work[1];
			//Tau=Cblk->rpar[0];
			TauInv=1./Tau;

			//sciprint("Tau=%f\n", Tau);			
			
			for(i=0; i<n; ++i) xd[i] = IN/Tau;
			//for(i=0; i<n; ++i) xd[i] = in[i]*TauInv;
			break;

		case ReInitialization:
		case OutputUpdate:	
			// Evaluation de la fonction F
			//----------------------------
			x = GetState(Cblk);
			out = GetRealOutPortPtrs(Cblk,1);
			
			

			for(i=0; i<n; ++i) {
							x[i] = Limiteur(x[i],sature,UL,LL); 	// Limitation de la matrice d'état
							if(fabs(x[i])<1.e-15) {
													x[i]=0.;			// Saturation - Rajout PPe le 18.06.2019
							}
							if (x[i]<=0.) {
													work = (double *)GetWorkPtrs(Cblk);
													work[0]=0.;								
							}
							out[i]=x[i];
			}
			break;
			
		case StateUpdate:
			// Evaluation de la fonction Dx
			//-----------------------------
			t = 0;//get_scicos_time();				// time en unité seconde				
			
			nevptr=GetNevIn(Cblk);
			
			if (nevptr & 1) {
							sciprint("Désaturation rapide de l'intégrateur EV1...time=%f \n", t);
							x = GetState(Cblk);
							if (x[0]>0.) {
											work = (double *)GetWorkPtrs(Cblk);
											work[0]=1.;
											work[1]=x[0];
							}
			}
			
			if (nevptr & 2) {
							sciprint("Reset de l'intégrateur EV2...time=%f \n", t);
			
							x = GetState(Cblk);
							in2 = GetRealInPortPtrs(Cblk,2); 
							for(i=0; i<n; ++i) x[i] = Limiteur(*in2,sature,UL,LL);	// Modif PPe le 24.08.2021 in2 devient un scalaire 1x1
			}
			break;
			
		case Ending:
					FREE(GetWorkPtrs(Cblk));
					break;				
			
	} 
}