//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	DiracSync
// Auteur			:	PERRICHON Pierre
// Date				:	07/01/2021
// Version			:	1.0
//
// Sujet			: 	Génération de peigne de Dirac synchronisé sur une date d'amorçage
//
// Paramétrage		:	Instant de départ (s)
//						ADirac
//						Période
//						Durée de l'impulstion (mettre typiquement 100 ms)
//						Impulsion sysmétrique (1) ou non (0) à chaue période
//
// Autres données	:	
//						
//						
// Sorties			:	Peigne de Dirac
//						
//						
//						
// Interface IHM	:	DiracSync.sci
// Code objet		;	DiracSyncC.c 
//
// Programme de test:	DiracSync.zos, GeneBF.zcos
//
//------------------------------------------------------------------------------------------------------------------------
#include <math.h>
#include "scicos_block4.h"
#include "scicos.h"

#include "version.h"
#if SCI_VERSION_MAJOR < 6
#include "MALLOC.h"
#else
#include "sci_malloc.h"
#endif

//------------------------------------------------------------------------------------------------------------------------

double GeneBF_SinuSyncC(scicos_block *Cblk, scicos_flag flag)
{
	static char *Module="GeneBF_SinuSyncC";
	
	double Out=0.;
	double tSinus, ASinus, PSinus, omega,pi;
	double t;
			
	switch (flag) {
								
		case ReInitialization:
		case OutputUpdate:
		
					tSinus=Cblk->rpar[5];
					ASinus=Cblk->rpar[6];
					PSinus=Cblk->rpar[7];
					
					t = get_scicos_time();
					 
					pi=M_PI;
					omega=2.*pi/PSinus;
					
					Out=t<tSinus ? 0. : ASinus*sin(omega*(t-tSinus));
					
					break;											
	}
	return Out;
}
//------------------------------------------------------------------------------------------------------------------------
double GeneBF_Step(scicos_block *Cblk, scicos_flag flag)
{
		static char *Module="GeneBF_Step";

		double t;
		double Out=0;
		double dw1, tdw1, dw2, tdw2;
		
		dw1=Cblk->rpar[1];
		tdw1=Cblk->rpar[2];
		dw2=Cblk->rpar[3];
		tdw2=Cblk->rpar[4];
		
		
		switch (flag) {
		
			case ReInitialization:
			case OutputUpdate:
		
					t=get_scicos_time();
					
					if (t>=tdw1) Out+=dw1;
					
					if (t>=tdw2) Out+=dw2;
							
					break;
			
		}
		return Out;		
}
//------------------------------------------------------------------------------------------------------------------------
void GeneBFC(scicos_block *Cblk, scicos_flag flag)
{
	static char *Module="GeneBFC";
	
	double *Out;
	double AddSignal;
	double modPer;
	double tDirac, ADirac, PDirac, DDirac, SDirac, difTime;
	long int idifTime,iPDirac,iTime;
	double t;
	
	long int *work = NULL;	
	int flipflop;
	
	modPer=Cblk->rpar[0];
	
	if (modPer==0.) AddSignal=GeneBF_Step(Cblk, flag);
	
	 else AddSignal=GeneBF_SinuSyncC(Cblk, flag);
			
	switch (flag) {
		
		case Initialization:
					// Allocation d'une zône de travail de 1 doubles
					//----------------------------------------------
					if ((GetWorkPtrs(Cblk) = (double *)MALLOC(2*sizeof(long int)))== NULL ) {
						Coserror("%s: Memory allocation problem\n", "LocalMax");
						return;
					}					
					work = (double *)GetWorkPtrs(Cblk);
					
					work[0] = 0;		// Valeur d'initialisation de la mémoire
					work[1] = -1;		// Valeur d'initialisation de la mémoire

					break;	
		
								
		case ReInitialization:
		case OutputUpdate:
		
					tDirac=Cblk->rpar[8];
					ADirac=Cblk->rpar[9];
					PDirac=Cblk->rpar[10];
					DDirac=Cblk->rpar[11];
					SDirac=Cblk->rpar[12];
					
					Out = GetRealOutPortPtrs(Cblk,1);
					
					t = get_scicos_time();				// t en unité seconde				
					
					if (t<tDirac) {
									Out[0]= AddSignal;
									break;
					}
					
					work = (double *)GetWorkPtrs(Cblk);
					

					t=t-tDirac;
					
					iTime=(double) t*1000.;				// Précision ms
					iPDirac=(double) PDirac*1000.;
					
					idifTime=iTime % iPDirac;
					difTime=(double) idifTime/1000.;	// Seconde
					 
					// Evaluation de la fonction F
					//----------------------------
					
					if (difTime<DDirac) {
						
							if (work[0]!=work[1])  {
													// Comptage des fronts montants
													//-----------------------------
													work[0]++;
													work[1]=work[0];
							}
							
							flipflop= SDirac==1 ? work[0] % 2 : 1;
							
							Out[0]= flipflop==0 ? -ADirac : ADirac;							
						
					}
					else {
							Out[0]=0;
							work[1]=0;
					}
					
					Out[0]=Out[0] + AddSignal;

					break;	

		case Ending:
					FREE(GetWorkPtrs(Cblk));
					break;				

	}
}